<?php

namespace Drupal\auto\Adapter;

use Drupal\auto\Entity\ContentEntityExtended;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Class AdapterBase.
 */
abstract class AdapterBase {
  
  const HAS_ONE_REFERENCED = 1;
  
  const HAS_ONE_REFERENCING = 2;
  
  const HAS_MANY_REFERENCED = 10;
  
  const HAS_MANY_REFERENCING = 20;
  
  /**
   * Entità da adattare.
   * 
   * @var \Drupal\auto\Entity\ContentEntityExtended
   */
  protected $entity;
  
  public function __construct(ContentEntityExtended $entity) {
    $this->entity = $entity;
  }
  
  /**
   * Implementazione del metodo magico get.
   */
  public function __get($name) {
    if ($name == 'entity') {
      return $this->entity;
    }
    $relationships = $this->relationships();
    if (isset($relationships[$name])) {
      $relationship = $relationships[$name];
      switch ($relationship[0]) {
        case static::HAS_ONE_REFERENCED:
          return $this->getSingleReferenced($name);
          break;
        case static::HAS_ONE_REFERENCING:
          return $this->getSingleReferencing($name,  $relationship[1]);
          break;
        case static::HAS_MANY_REFERENCED:
          return $this->getMultipleReferenced($name);
          break;
        case static::HAS_MANY_REFERENCING:
          return $this->getMultipleReferencing($name, $relationship[1]);
          break;
      }
    }
    else {
      $attributes = $this->attributes();
      if (isset($attributes[$name]) && !empty($attributes[$name]['multiple'])) {
        return array_map(function ($val) { return $val['value']; }, $this->entity->get($name)->getValue());
      }
      else {
        return $this->entity->get($name)->getString();
      }
    }
  }
  
  /**
   * Restituisce l'elenco degli attributi dell'entità adattata.
   * 
   * @return array
   */
  protected function attributes() {
    return [];
  }
  
  /**
   * Restituisce l'elenco delle relazioni dell'entità adattata.
   * 
   * @return array
   */
  protected function relationships() {
    return [];
  }
  
  /**
   * Restituisce una entità singola riferita.
   * 
   * @param string $entity_id
   * @return \Drupal\auto\Adapter\AdapterBase|NULL
   */
  protected function getSingleReferenced($entity_id) {
    $return = NULL;
    if ($firstReferenced = $this->entity->get($entity_id)->first()) {
      $return = $firstReferenced->get('entity')->getTarget()->getValue()->getAdapter();
    }
    return $return;
  }
  
  /**
   * Restituisce una entità singola che si riferisce a quella dell'adattatore.
   * 
   * @param string $entity_id
   * @param string $referencing_field
   * @return \Drupal\auto\Adapter\AdapterBase|NULL
   */
  protected function getSingleReferencing($entity_id, $referencing_field) {
    $return = NULL;
    $searchValues = [
      $referencing_field => $this->entity->id(),
    ];
    $entityList = \Drupal::service('entity_type.manager')->getStorage($entity_id)->loadByProperties($searchValues);
    if (!empty($entityList)) {
      $entity = array_shift($entityList);
      $return = $entity->getAdapter();
    }
    return $return;
  }
  
  /**
   * Restituisce una lista di entità riferite.
   * 
   * @param string $entity_id
   * @return \Drupal\auto\Adapter\AdapterBase[]
   */
  protected function getMultipleReferenced($entity_id) {
    $return = [];
    foreach ($this->entity->get($entity_id) as $referenced) {
      $return[] = $referenced->get('entity')->getTarget()->getValue()->getAdapter();
    }
    return $return;
  }
  
  /**
   * Restituisce una lista do entità che si riferiscono a quella dell'adattatore.
   * 
   * @param string $entity_id
   * @param string $referencing_field
   * @return \Drupal\auto\Adapter\AdapterBase[]
   */
  protected function getMultipleReferencing($entity_id, $referencing_field) {
    $return = [];
    $searchValues = [
      $referencing_field => $this->entity->id(),
    ];
    $entityList = \Drupal::service('entity_type.manager')->getStorage($entity_id)->loadByProperties($searchValues);
    if (!empty($entityList)) {
      foreach ($entityList as $entity) {
        $return[] = $entity->getAdapter();
      }
    }
    return $return;
  }
  
}
