<?php

namespace Drupal\auto\Entity;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItem;
use Drupal\user\UserInterface;

/**
 * Defines the Auto entity.
 *
 * @ingroup auto
 *
 * @ContentEntityType(
 *   id = "auto",
 *   label = @Translation("Auto"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\auto\AutoListBuilder",
 *     "views_data" = "Drupal\auto\Entity\AutoViewsData",
 *
 *     "form" = {
 *       "default" = "Drupal\auto\Form\AutoForm",
 *       "add" = "Drupal\auto\Form\AutoForm",
 *       "edit" = "Drupal\auto\Form\AutoForm",
 *       "delete" = "Drupal\auto\Form\AutoDeleteForm",
 *     },
 *     "access" = "Drupal\auto\AutoAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\auto\AutoHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "auto",
 *   admin_permission = "administer auto entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "modello",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "langcode" = "langcode",
 *     "status" = "status",
 *   },
 *   links = {
 *     "canonical" = "/auto/auto/{auto}",
 *     "add-form" = "/auto/auto/add",
 *     "edit-form" = "/auto/auto/{auto}/edit",
 *     "delete-form" = "/auto/auto/{auto}/delete",
 *     "collection" = "/auto/auto",
 *   },
 *   field_ui_base_route = "auto.settings"
 * )
 * 
 * @property \Drupal\auto\Adapter\AutoAdapter $adapter
 */
class Auto extends ContentEntityExtended implements AutoInterface {

  use EntityChangedTrait;
  
  /**
   * {@inheritdoc}
   */
  protected function adapterClass(): string {
    return '\Drupal\auto\Adapter\AutoAdapter';
  }

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += [
      'user_id' => \Drupal::currentUser()->id(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isPublished() {
    return (bool) $this->getEntityKey('status');
  }

  /**
   * {@inheritdoc}
   */
  public function setPublished($published) {
    $this->set('status', $published ? TRUE : FALSE);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of author of the Auto entity.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

//    $fields['name'] = BaseFieldDefinition::create('string')
//      ->setLabel(t('Name'))
//      ->setDescription(t('The name of the Auto entity.'))
//      ->setSettings([
//        'max_length' => 50,
//        'text_processing' => 0,
//      ])
//      ->setDefaultValue('')
//      ->setDisplayOptions('view', [
//        'label' => 'above',
//        'type' => 'string',
//        'weight' => -4,
//      ])
//      ->setDisplayOptions('form', [
//        'type' => 'string_textfield',
//        'weight' => -4,
//      ])
//      ->setDisplayConfigurable('form', TRUE)
//      ->setDisplayConfigurable('view', TRUE);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Publishing status'))
      ->setDescription(t('A boolean indicating whether the Auto is published.'))
      ->setDefaultValue(TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));
    
    $fields['marca'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Marca'))
      ->setDescription(t('Marca dell\'Auto.'))
      ->setRequired(TRUE)
      ->setSettings([
        'max_length' => 50,
        'text_processing' => 0,
      ])
      ->setDefaultValue('')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -200,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -200,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);
    
    $fields['modello'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Modello'))
      ->setDescription(t('Modello dell\'Auto.'))
      ->setRequired(TRUE)
      ->setSettings([
        'max_length' => 50,
        'text_processing' => 0,
      ])
      ->setDefaultValue('')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -100,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -100,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);
    
    $fields['ruote'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Ruote'))
      ->setDescription(t('Riferimento alle ruote.'))
      ->setRequired(TRUE)
      ->setCardinality(4)
      ->setSetting('target_type', 'ruota')
      ->setSetting('handler', 'default')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'weight' => 100,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 100,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);
    
    $fields['conducenti'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Conducenti'))
      ->setDescription(t('Riferimento ai conducenti.'))
      ->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED)
      ->setSetting('target_type', 'conducente')
      ->setSetting('handler', 'default')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'weight' => 200,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 200,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);
    
    $fields['date_tagliandi'] = BaseFieldDefinition::create('datetime')
      ->setLabel(t('Date tagliandi'))
      ->setDescription(t('Le date in cui sono stati eseguiti gli interventi di manutenzione obbligatoria.'))
      ->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED)
      ->setSettings([
        'max_length' => 50,
        'text_processing' => 0,
        'datetime_type' => DateTimeItem::DATETIME_TYPE_DATE,
      ])
      ->setDefaultValue(gmdate(DATETIME_DATE_STORAGE_FORMAT, time()))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => 300,
        'type' => 'datetime_default',
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 300,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

}
